; ---------------------- LAYER ON FUNCTION -----------------------
; Turns all layers on
; ----------------------------------------------------------------
 
(Defun C:ON ()
 
  (acet-error-init
    (list
      (list "cmdecho" 0)
      nil     ;flag. True means use undo for error clean up.
    );list
  );acet-error-init
 
  (Command "_.-LAYER" "_ON" "*" "")
  (prompt "\nAll layers have been turned on.")
 
  (acet-error-restore)
 
  (princ)
)

; --------------- CHANGE TO CURRENT LAYER FUNCTION ---------------
; Changes the layer of selected object(s) to the current layer
; ----------------------------------------------------------------
 
(Defun C:cg (/ SS CNT LAY)
 
  (acet-error-init
    (list
      (list "cmdecho" 0)
      T     ;flag. True means use undo for error clean up.
    );list
  );acet-error-init
 
 
  (if (not (setq SS (ssget "_i")))
    (progn
      (prompt "\nSelect objects to be changed to the current layer: ")
      (setq SS (ssget))
    )
  )
 
  (if SS
    (progn
      (setq CNT (sslength SS))
      (princ (acet-str-format "\n%1 found."  (itoa CNT) ))       ; Report number of items found
      (sssetfirst nil nil)
      (command "_.move" SS "")                         ; filter out objects on locked layers
 
      (if (> (getvar "cmdactive") 0)                   ; if there are still objects left
        (progn
          (command "0,0" "0,0")
          (setq SS  (ssget "_p")
                CNT (- CNT (sslength SS))              ; count them
          )
        )
        (setq SS nil)                                  ; else abort operation
      )
 
      (if (> CNT 0)                                    ; if items where filtered out
        (if (= CNT 1)
          (princ (acet-str-format "\n%1 was on a locked layer."  (itoa CNT) ))   ; report it.
          (princ (acet-str-format "\n%1 were on a locked layer."  (itoa CNT) ))
        )
      )
    )
  )
 
  (if SS
    (progn
      (setq LAY (getvar "CLAYER"))
 
      (command "_.chprop" SS "" "_la" LAY "")
 
      (if (= (sslength SS) 1)
        (prompt (acet-str-format "\nOne object changed to layer %1 (the current layer)."  LAY ))
        (prompt (acet-str-format "\n%1 objects changed to layer %2 (the current layer)." (itoa (sslength SS)) LAY))
      )
    )
  )
 
  (acet-error-restore)
 
  (princ)
)


; -------------------- ISOLATE LAYER FUNCTION --------------------
; Isolates selected object's layer by turning all other layers off
; ----------------------------------------------------------------
 
(Defun C:ff (/ SS CNT LAY LAYLST VAL VPMODE)
  ;;;local function to find an unused layer state name for saving
 
  (acet-error-init
    (list
      (list "cmdecho" 0
            "expert"  0
      )
      T     ;flag. True means use undo for error clean up.
    );list
  );acet-error-init
  (acet-autoload '("lman.lsp" "(bns_sl  name)"))   ;;changed to support LAYUNISO
 
  (if (and (= 1 (acet-LayVpMode-Get))
           (= 0 (getvar "tilemode"))                          ; if in a paper space
           (/= 1 (getvar "cvport"))                           ; viewport
      )
      (setq VPMODE T)                                           ; set flag for viewport behavior
  )
 
  (if (not (setq SS (ssget "_i")))
    (progn
      (prompt "\nSelect object(s) on the layer(s) to be isolated: ")
      (setq SS (ssget))
    )
  )
 
 
  (if SS
    (progn
 
      (setq CNT 0)
 
      (while (and ss
                  (> (sslength ss) 0)
                  (setq LAY (ssname SS CNT))
             );and
        (setq LAY (cdr (assoc 8 (entget LAY))))
        (if (not (member LAY LAYLST))
          (setq LAYLST (cons LAY LAYLST))
        )
        (setq CNT (1+ CNT))
      )
 
      (cond
        (VPMODE
          (bns_sl "ACET-lAYISO") ;setq    Save the current layer state
          (setq acet:LSTVPORT (list (getvar "ctab") (acet-currentviewport-ename))) ;setq save the layout and the viewport
          (command "_.vplayer" "_freeze" "*" "_Current")
          (foreach VAL LAYLST (command "_Thaw" VAL "_Current"))
          (command "")
          (if (= (length LAYLST) 1)
            (prompt (acet-str-format "\nLayer %1 has been isolated in this viewport." (car LAYLST)))
            (prompt (acet-str-format "\n%1 layers have been isolated in this viewport." (itoa (length LAYLST))))
          )
        )
        (T
          (bns_sl "acet-layiso") ;setq    Save the current layer state
          (setq acet:LSTVPORT (list (getvar "ctab") (acet-currentviewport-ename))) ;setq save the layout and the viewport
          ;(setq acet:LSTVPORT (list "Model")) ;setq   new GLOBAL for LAYUNISO
          (if (member (getvar "CLAYER") LAYLST)
            (setq LAY (getvar "CLAYER"))
            (setvar "CLAYER" (setq LAY (last LAYLST)))
          )
 
          (command "_.-LAYER" "_OFF" "*" "_Y")
          (foreach VAL LAYLST (command "_ON" VAL))
          (command "")
 
          (if (= (length LAYLST) 1)
            (prompt (acet-str-format "\nLayer %1 has been isolated." (car LAYLST)))
            (prompt (acet-str-format "\n%1 layers have been isolated. Layer %2 is current."
                                      (itoa (length LAYLST)) LAY)
            )
          )
        )
      )
    )
  )
 
  (acet-error-restore)
 
  (princ)
)
 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Controls usage of vplayer vs layer when in paper space.
;
(defun c:layvpmode ()
 (acet-error-init nil)
 (acet-LayVpMode-ui-get)
 (acet-error-restore)
);defun c:layvpmode
 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun acet-LayVpMode-Get ( / a )
 (setq a (acet-getvar (list "ACET-LAYVPMODE")))
 (if (or (not a)
         (not (equal 'INT (type a)))
     );or
     (progn
      (acet-setvar (list "ACET-LAYVPMODE" 1 3))
      (setq a 1)
     );progn then
 );if
 a
);defun acet-Layvpmode-Get
 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun acet-LayVpMode-Set ( a / )
 (if (and a
          (equal 'INT (type a))
     );and
     (progn
      (acet-setvar (list "ACET-LAYVPMODE" a 3))
     );progn then
     (setq a nil)
 );if
 a
);defun acet-LayVpMode-Set
 
 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Prompts for layer viewport mode. Vpfreeze or freeze/off
;
(defun acet-LayVpMode-ui-get ( / ans def )
 (setq def (acet-LayVpMode-get))
 (initget "Yes No")
 (if (= def 1)
     (setq ans (getkword "\nUse 'VpFreeze' in place of 'Freeze' or 'Off' when in paper space? <Y>: "))
     (setq ans (getkword "\nUse 'VpFreeze' in place of 'Freeze' or 'Off' when in paper space? <N>: "))
 );if
 (if ans
     (progn 
      (if (= ans "Yes")
          (setq def 1)
          (setq def 0)
      );if
      (acet-LayVpMode-Set def)
     );progn then
 );if
 def
);defun acet-LayVpMode-ui-get
